@@ -132,7 +132,7 @@ class LocalFilesystemBlobAccess(ResourceAccess):
 @dc(frozen=True)
 class Label:
     name: str
-    value: typing.Union[str, int, float, bool, dict]
+    value: typing.Union[str, int, float, bool, dict, list]
 
 
 _no_default = object()
@@ -171,6 +171,22 @@ class LabelMethodsMixin:
             labels=patched_labels,
         )
 
+@dc
+class DigestSpec:
+    hashAlgorithm: str
+    normalisationAlgorithm: str
+    value: str
+
+@dc
+class SignatureSpec:
+    algorithm: str
+    value: str
+
+@dc
+class Signature:
+    name: str
+    digest: DigestSpec
+    signature: SignatureSpec
 
 class Provider(enum.Enum):
     '''
@@ -311,6 +327,7 @@ class ComponentReference(Artifact, LabelMethodsMixin):
     name: str
     componentName: str
     version: str
+    digest: typing.Optional[DigestSpec] = None
     extraIdentity: typing.Dict[str, str] = dataclasses.field(default_factory=dict)
     labels: typing.List[Label] = dataclasses.field(default_factory=tuple)
 
@@ -327,21 +344,24 @@ class Resource(Artifact, LabelMethodsMixin):
     version: str
     type: typing.Union[ResourceType, str]
     access: typing.Union[
-        OciAccess,
-        RelativeOciAccess,
+        # Order of types is important for deserialization. The first matching type will be taken,
+        # i.e. keep generic accesses at the bottom of the list
         GithubAccess,
         HttpAccess,
-        LocalOCIBlobAccess,
         LocalFilesystemBlobAccess,
+        LocalOCIBlobAccess,
+        OciAccess,
+        RelativeOciAccess,
+        S3Access,
         ResourceAccess,
         None,
     ]
+    digest: typing.Optional[DigestSpec] = None
     extraIdentity: typing.Dict[str, str] = dataclasses.field(default_factory=dict)
     relation: ResourceRelation = ResourceRelation.LOCAL
     labels: typing.List[Label] = dataclasses.field(default_factory=tuple)
     srcRefs: typing.List[SourceReference] = dataclasses.field(default_factory=tuple)
 
-
 @dc(frozen=True)
 class RepositoryContext:
     pass  # actually, must have attr `type`
@@ -409,6 +429,7 @@ def enum_or_string(v, enum_type: enum.Enum):
 class ComponentDescriptor:
     meta: Metadata
     component: Component
+    signatures: typing.List[Signature] = dataclasses.field(default_factory=list)
 
     @staticmethod
     def validate(
